import { useEffect, useMemo, useRef, useState } from "react";
import { Button, Select, Spin } from "antd";
import type { SelectProps } from "antd";
import { api } from "@nice/client";
import React from "react";
import { SizeType } from "antd/es/config-provider/SizeContext";
import "./StaffSelect.css";
interface StaffSelectProps {
	value?: string | string[];
	onChange?: (value: string | string[]) => void;
	style?: React.CSSProperties;
	multiple?: boolean;
	domainId?: string;
	placeholder?: string;
	limitedIds?: string[];
	size?: SizeType;
}

export default function StaffSelect({
	value,
	onChange,
	placeholder,
	style,
	multiple,
	limitedIds,
	domainId,
	size,
}: StaffSelectProps) {
	const [keyword, setQuery] = useState<string>("");

	// Determine ids based on whether value is an array or not
	const ids = useMemo(() => {
		return Array.isArray(value) ? value : [];
	}, [value]);

	// Adjust the query to include ids when they are present
	const { data, isLoading } = api.staff.findMany.useQuery({
		where: {
			id: limitedIds?.length > 0 ? { in: limitedIds } : undefined,
			deletedAt: null,
			OR: [
				{
					username: {
						contains: keyword,
					},
				},
				{
					showname: {
						contains: keyword,
					},
				},
				{
					id: {
						in: ids,
					},
				},
			],
			domainId,
		},
		select: { id: true, showname: true, username: true },
		take: 30,
		orderBy: { order: "asc" },
	});

	const handleSearch = (value: string) => {
		setQuery(value);
	};

	const options: SelectProps["options"] =
		data?.map((staff: any) => ({
			value: staff.id,
			label: staff?.showname || staff?.username,
		})) || [];
	const [isOpen, setIsOpen] = useState(false);
	const dropdownRef = useRef<HTMLDivElement>(null);
	// 检测旧版本浏览器
	const isLegacyBrowser = () => {
		const chromeMatch = navigator.userAgent.match(/Chrome\/(\d+)/);
		if (chromeMatch) {
			const version = parseInt(chromeMatch[1], 10);
			return version <= 87;
		}
		return false;
	}; // 添加点击外部关闭的处理
	useEffect(() => {
		const handleClickOutside = (event: MouseEvent) => {
			if (
				dropdownRef.current &&
				!dropdownRef.current.contains(event.target as Node)
			) {
				setIsOpen(false);
			}
		};

		// 只在下拉框打开时添加事件监听
		if (isOpen) {
			document.addEventListener("mousedown", handleClickOutside);
		}

		return () => {
			document.removeEventListener("mousedown", handleClickOutside);
		};
	}, [isOpen]); // 依赖项添加 isOpen

	useEffect(() => {
		if (isLegacyBrowser()) {
			const handleClickOutside = (event: MouseEvent) => {
				if (
					dropdownRef.current &&
					!dropdownRef.current.contains(event.target as Node)
				) {
					setIsOpen(false);
				}
			};
			document.addEventListener("mousedown", handleClickOutside);
			return () =>
				document.removeEventListener("mousedown", handleClickOutside);
		}
	}, []);

	if (isLegacyBrowser()) {
		return (
			<div
				className={`legacy-staff-select ${size === "small" ? "small" : ""}`}
				ref={dropdownRef}
				style={{ ...style }}>
				<div className="legacy-staff-input-wrapper">
					<input
						type="text"
						className={`legacy-staff-search ${
							(multiple && value?.length > 0) ||
							(!multiple && value)
								? "has-value"
								: ""
						}`}
						placeholder={
							multiple
								? value?.length > 0
									? Array.isArray(value) &&
										value.length > 0 &&
										(data as any)
											?.map((staff) => {
												if (value.includes(staff.id)) {
													return (
														staff.showname ||
														staff.username
													);
												}
												return null;
											})
											?.filter(Boolean) // 显示已选择的数量
									: placeholder || "请选择"
								: value
									? (data as any)?.find(
											(staff) => staff.id === value
										)?.showname
									: placeholder || "请选择"
						}
						value={keyword}
						onChange={(e) => setQuery(e.target.value)}
						onClick={() => setIsOpen(true)}
					/>
					{/* {multiple ? (
						<div className="legacy-staff-tags">
							{Array.isArray(value) &&
								value.length > 0 &&
								(data as any)?.map((staff) => {
									if (value.includes(staff.id)) {
										return (
											<span
												key={staff.id}
												className="legacy-staff-tag">
												{staff.showname ||
													staff.username}
												<span
													className="legacy-staff-tag-close"
													onClick={(e) => {
														e.stopPropagation();
														const newValue = (
															value as string[]
														).filter(
															(v) =>
																v !== staff.id
														);
														onChange?.(newValue);
													}}>
													×
												</span>
											</span>
										);
									}
									return null;
								})}
						</div>
					) : null} */}
				</div>

				{isOpen && (
					<div className="legacy-staff-dropdown">
						{isLoading ? (
							<div className="legacy-staff-loading">
								<Spin size="small" />
							</div>
						) : data?.length === 0 ? (
							<div className="legacy-staff-empty">无匹配结果</div>
						) : (
							(data as any)?.map((staff) => (
								<div
									key={staff.id}
									className={`legacy-staff-option ${
										multiple
											? Array.isArray(value) &&
												value.includes(staff.id)
												? "selected" // 添加 selected 类名
												: ""
											: value === staff.id
												? "selected"
												: ""
									}`}
									onClick={() => {
										if (multiple) {
											const newValue = Array.isArray(
												value
											)
												? value.includes(staff.id)
													? value.filter(
															(v) =>
																v !== staff.id
														)
													: [...value, staff.id]
												: [staff.id];
											onChange?.(newValue);
										} else {
											onChange?.(staff.id);
											setIsOpen(false);
										}
									}}>
									{staff.showname || staff.username}
								</div>
							))
						)}
					</div>
				)}
			</div>
		);
	}

	return (
		<>
			<Select
				size={size}
				allowClear
				showSearch
				mode={multiple ? "multiple" : undefined}
				placeholder={placeholder || "请选择人员"}
				notFoundContent={isLoading ? <Spin size="small" /> : null}
				filterOption={false}
				onSearch={handleSearch}
				options={options}
				value={value}
				onChange={onChange}
				style={{
					minWidth: 200,
					position: "relative", // 确保定位正常
					display: "inline-block",
					...style,
				}}
				// 添加以下属性来优化显示
				dropdownStyle={{
					maxHeight: "400px",
					overflow: "auto",
					minWidth: "200px",
				}}
				// 添加类名以便自定义样式
				className="custom-staff-select"
			/>{" "}
		</>
	);
}
